<?php
  /**
   * @author Matthew McNaney
   * @version $Id: update.php 5472 2007-12-11 16:13:40Z jtickle $
   */

function filecabinet_update(&$content, $version)
{
    $home_dir = PHPWS_Boost::getHomeDir();
    switch ($version) {
    case version_compare($version, '1.0.1', '<'):
        $content[] = '<pre>File Cabinet versions prior to 1.0.1 are not supported.
Please download version 1.0.2.</pre>';
        break;

    case version_compare($version, '1.0.2', '<'):
        $content[] = '<pre>';
        $db = new PHPWS_DB('folders');
        if (!$db->isTableColumn('key_id')) {
            if (PHPWS_Error::logIfError($db->addTableColumn('key_id', 'int NOT NULL default 0'))) {
                $content[] = '--- An error occurred when trying to add key_id as a column to the folders table.</pre>';
                return false;
            }
            $content[] = '--- Successfully added key_id column to folders table.';

            $db2 = new PHPWS_DB('phpws_key');
            $db2->addWhere('module', 'filecabinet');
            $db2->delete();
            $content[] = '--- Deleted false folder keys.';

            $db->reset();
            PHPWS_Core::initModClass('filecabinet', 'Folder.php');
            $result = $db->getObjects('Folder');
            if (!empty($result)) {
                foreach ($result as $folder) {
                    $folder->saveKey(true);
                }
            }
        }
        $content[] = '
1.0.2 changes
--------------
+ 1.0.0 update was missing key_id column addition to folders table.
</pre>';


    case version_compare($version, '1.1.0', '<'):
        $content[] = '<pre>';
        if (!is_dir($home_dir . 'files/multimedia')) {
            if (is_writable($home_dir . 'files/') && @mkdir($home_dir . 'files/multimedia')) {
                $content[] = '--- "files/multimedia" directory created.';
            } else {
                $content[] = 'File Cabinet 1.1.0 requires the creation of a "multimedia" directory.
Please place it in the files/ directory.
Example: mkdir phpwebsite/files/multimedia/</pre>';
                return false;
            }
        } elseif (!is_writable($home_dir . 'files/multimedia')) {
            $content[] = 'Your files/multimedia directory is not writable by the web server.
 Please change its permissions and return.</pre>';
            return false;
        }

        if (!is_dir($home_dir . 'files/filecabinet/incoming')) {
            if (is_writable($home_dir . 'files/filecabinet') && @mkdir($home_dir . 'files/filecabinet/incoming')) {
                $content[] = '--- "files/filecabinet/incoming" directory created.';
            } else {
                $content[] = 'File Cabinet 1.1.0 is unable to create a "filecabinet/incoming" directory.
It is not required but if you want to classify files you will need to create it yourself.
Example: mkdir phpwebsite/files/filecabinet/incoming/</pre>';
                return false;
            }
        }

        $source_dir = PHPWS_SOURCE_DIR . 'mod/filecabinet/templates/filters/';
        $dest_dir   = $home_dir . 'templates/filecabinet/filters/';

        if (!is_dir($dest_dir)) {
            if (!PHPWS_File::copy_directory($source_dir, $dest_dir)) {
                $content[] = '--- FAILED copying templates/filters/ directory locally.</pre>';
                return false;
            }
        }

        $files = array('templates/manager/pick.tpl', 'templates/classify_file.tpl', 
                       'templates/classify_list.tpl', 'templates/image_edit.tpl', 
                       'templates/multimedia_edit.tpl', 'templates/multimedia_grid.tpl',
                       'templates/style.css', 'templates/settings.tpl', 'conf/config.php');
        
        if (PHPWS_Boost::updateFiles($files, 'filecabinet')) {
            $content[] = '--- Copied the following files:';
        } else {
            $content[] = '--- FAILED copying the following files:';
        }

        $content[] = "    " . implode("\n    ", $files);

        $db = new PHPWS_DB('images');
        if (!$db->isTableColumn('parent_id')) {
            if (PHPWS_Error::logIfError($db->addTableColumn('parent_id', 'int NOT NULL default 0'))) {
                $content[] = 'Could not create parent_id column in images table.</pre>';
                return false;
            }
        }

        if (!$db->isTableColumn('url')) {
            if (PHPWS_Error::logIfError($db->addTableColumn('url', 'varchar(255) NULL'))) {
                $content[] = 'Could not create url column in images table.</pre>';
                return false;
            }
        }

       if (!PHPWS_DB::isTable('multimedia')) {
            $result = PHPWS_DB::importFile(PHPWS_SOURCE_DIR . 'mod/filecabinet/boost/multimedia.sql');
            if (!PHPWS_Error::logIfError($result)) {
                $content[] = '--- Multimedia table created successfully.';
            } else {
                $content[] = '--- Failed to create multimedia table.</pre>';
                return false;
            }
        }
        
        $content[] = '
1.1.0 changes
--------------
+ Fixed authorized check when unpinning folders
+ Images can now be linked to other pages.
+ Resized images can now be linked to their parent image.
+ Clip option moved outside edit_folder permissions when viewing images.
+ Added writable directory check before allowing new folders to be
  created.
+ Fixed some error messages in File_Common.
+ Commented out ext variable in File_Common. Doesn\'t appear to be in
  use.
+ Created setDirectory function for File_Common. Assures trailing
  forward slash on directory name.
+ Removed itemname variable from Document_Manager
+ Added ability to classify uploaded files.
+ New folder class - Multimedia
+ Multimedia files can be clipped and pasted via SmartTags.
</pre>
';

    case version_compare($version, '1.2.0', '<'):
        $content[] = '<pre>';
        $files = array('img/no_image.png', 'conf/config.php', 'conf/video_types.php',
                       'conf/embedded.php',
                       'javascript/folder_contents/head.js',
                       'javascript/clear_image/head.js',
                       'javascript/clear_image/body.js',
                       'javascript/pick_image/head.js',
                       'templates/image_folders.tpl', 'templates/settings.tpl',
                       'templates/style.css', 'templates/image_view.tpl',
                       'templates/multimedia_view.tpl', 'templates/style.css',
                       'img/video_generic.png', 'templates/image_edit.tpl', 'conf/error.php');

        fc_updatefiles($files, $content);

        if (!PHPWS_Boost::inBranch()) {
            $content[] = file_get_contents(PHPWS_SOURCE_DIR . 'mod/filecabinet/boost/changes/1_2_0.txt');
        }
        $content[] = '</pre>';

    case version_compare($version, '1.2.1', '<'):
        $content[] = '<pre>';
        if (!PHPWS_DB::isTable('filecabinet_pins')) {
            $db = new PHPWS_DB('filecabinet_pins');
            $db->addValue('key_id', 'int not null default 0');
            $db->addValue('folder_id', 'int not null default 0');
            if (PHPWS_Error::logIfError($db->createTable())) {
                $content[] = 'Failed to create filecabinet_pins table.</pre>';
                return false;
            }
            $content[] = '--- Created filecabinet_pins table.';
        }

        $files = array('templates/settings.tpl');

        fc_updatefiles($files, $content);

        if (!PHPWS_Boost::inBranch()) {
            $content[] = file_get_contents(PHPWS_SOURCE_DIR . 'mod/filecabinet/boost/changes/1_2_1.txt');
        }
        $content[] = '</pre>';

    case version_compare($version, '1.2.2', '<'):
        $content[] = '<pre>';
        $files = array('templates/image_edit.tpl');
        fc_updatefiles($files, $content);

        if (!PHPWS_Boost::inBranch()) {
            $content[] = file_get_contents(PHPWS_SOURCE_DIR . 'mod/filecabinet/boost/changes/1_2_2.txt');
        }
        $content[] = '</pre>';

    case version_compare($version, '1.3.0', '<'):
        $content[] = '<pre>';
  
        $db = new PHPWS_DB('folders');
        if (!$db->isTableColumn('module_created')) {
            if (PHPWS_Error::logIfError($db->addTableColumn('module_created', 'varchar(40) default null'))) {
                $content[] = '--- Could not create column module_created on folders table.</pre>';
                return false;
            } else {
                $content[] = '--- Created module_created column on folders table.';
            }
        }


        $db = new PHPWS_DB('multimedia');
        $result = $db->addTableColumn('thumbnail', 'varchar(255) not null');
        if (PHPWS_Error::logIfError($result)) {
            $content[] = '--- Unable to add thumbnail column to multimedia table.</pre>';
            return false;
        } else {
            $content[] = '--- Added thumbnail column to multimedia table.';
        }

        $s1 = PHPWS_SOURCE_DIR . 'mod/filecabinet/templates/filters/flash/';
        $d1 = $home_dir . 'templates/filecabinet/filters/flash/';

        $s2 = PHPWS_SOURCE_DIR . 'mod/filecabinet/img/icons/';
        $d2 = $home_dir . 'images/mod/filecabinet/icons/';

        if (PHPWS_File::copy_directory($s1, $d1)) {
            $content[] = "--- Successfully copied $s1 to $d1";
        } else {
            $content[] = "--- Failed to copy $s1 to $d1</pre>";
            return false;
        }

        if (PHPWS_File::copy_directory($s2, $d2)) {
            $content[] = "--- Successfully copied $s2 to $d2";
        } else {
            $content[] = "--- Failed to copy $s2 to $d2</pre>";
            return false;
        }
        $content[] = '';
        $files = array('conf/error.php', 'conf/config.php',
                       'templates/filters/flash.tpl', 'templates/file_list.tpl',
                       'templates/multimedia_edit.tpl', 'templates/settings.tpl',
                       'templates/style.css', 'templates/thumbnail.tpl', 'templates/image_edit.tpl',
                       'javascript/pick_image/head.js', 'templates/folder_list.tpl',
                       'templates/manager/pick.tpl', 'img/delete.png');

        fc_updatefiles($files, $content);
        if (!PHPWS_Boost::inBranch()) {
            $content[] = file_get_contents(PHPWS_SOURCE_DIR . 'mod/filecabinet/boost/changes/1_3_0.txt');
        }
        $content[] = '</pre>';

    case version_compare($version, '1.4.0', '<'):
        $content[] = '<pre>';
        $files = array('javascript/folder_contents/head.js',
                       'javascript/folder_contents/scripts.js',
                       'javascript/pick_image/head.js',
                       'javascript/pick_image/scripts.js',
                       'javascript/clear_image/body.js',
                       'javascript/clear_image/head.js',
                       'templates/style.css',
                       'templates/settings.tpl');
        fc_updatefiles($files, $content);
        if (!PHPWS_Boost::inBranch()) {
            $content[] = file_get_contents(PHPWS_SOURCE_DIR . 'mod/filecabinet/boost/changes/1_4_0.txt');
        }
        $content[] = '</pre>';

  case version_compare($version, '1.4.1', '<'):
      $content[] = '<pre>';
      $db = new PHPWS_DB('folders');
      if (!$db->isTableColumn('module_created')) {
          if (PHPWS_Error::logIfError($db->addTableColumn('module_created', 'varchar(40) default null'))) {
              $content[] = '--- Could not create column module_created on folders table.</pre>';
              return false;
          } else {
              $content[] = '--- Created module_created column on folders table.';
          }
      }
      $content[] = '1.4.1 changes
--------------
+ module_created column missing from > 1.3.0 install.
</pre>';

  case version_compare($version, '1.4.2', '<'):
      $content[] = '<pre>
1.4.2 changes
--------------
+ Removed test echo 1
+ moved all defines to one file.
</pre>';
  
    }

    return true;
}

function fc_updatefiles($files, &$content)
{
    if (PHPWS_Boost::updateFiles($files, 'filecabinet')) {
        $content[] = '--- Copied the following files:';
    } else {
        $content[] = '--- FAILED copying the following files:';
    }

    $content[] = '    ' . implode("\n    ", $files);
    $content[] = '';
}


?>